import Vue from 'vue';

export default function (moduleApi, nested = false) {

  const getDefaultState = () => ({
    all: {}
  });

  const normalStore = {
    namespaced: true,
    state: getDefaultState(),
    getters: {
      getById: state => id => state.all[id],
      getAll: state => state.all,
    },
    mutations: {
      resetState(state) {
        Object.assign(state, getDefaultState());
      },
      freshSetAll(state, payload) {
        payload.data.forEach((item) => {
          Vue.set(state.all, item.id, item);
        });
      },
      setById(state, payload) {
        Vue.set(
          state.all, 
          payload.data.id, 
          Object.assign({}, state.all[payload.data.id], payload.data)
        );
      },
      removeById(state, payload) {
        Vue.delete(state.all, payload.id);
      },
    },
    actions: {
      resetState: {
        root: true,
        handler ({ commit }) {
          commit('resetState');
         }
      },
      async loadAll({ commit }, params = {}) {
        const response = await moduleApi.getAll(params);
        commit({ type: 'freshSetAll', data: response.data });
      },
      async loadById({ commit }, id) {
        const response = await moduleApi.getById(id);
        commit({ type: 'setById', data: response.data });
      },
      async updateById({ commit }, { id, payload }) {
        const response = await moduleApi.updateById(id, payload);
        commit({ type: 'setById', data: response.data });
      },
      async create({ commit }, payload) {
        const response = await moduleApi.create(payload);
        commit({ type: 'setById', data: response.data });
        return response;
      },
      async deleteById({ commit }, id) {
        await moduleApi.deleteById(id);
        commit({ type: 'removeById', id });
      },
    },
  };

  const nestedStore = {
    namespaced: true,
    state: getDefaultState(),
    getters: {
      getById: state => (firstId, secondId) => state.all[firstId][secondId],
      getAll: state => firstId => state.all[firstId],
    },
    mutations: {
      resetState(state) {
        Object.assign(state, getDefaultState());
      },
      setById(state, payload) {
        Vue.set(
          state.all, 
          payload.firstId,
          payload.data
        );
      },
      removeById(state, payload) {
        const index = state.all[payload.firstId].map(e => e.id).indexOf(payload.secondId);
        Vue.delete(state.all[payload.firstId], index);
      },
    },
    actions: {
      resetState: {
        root: true,
        handler ({ commit }) {
          commit('resetState');
         }
      },
      async loadAll({ commit }, { firstId }) {
        const response = await moduleApi.getAll(firstId);
        commit({ type: 'setById', firstId, data: response.data });
      },
      // async loadById({ commit }, { firstId, secondId }) {
      //   const response = await moduleApi.getById(firstId, secondId);
      //   commit({ type: 'setById', data: response.data });
      // },
      // async updateById({ commit }, { firstId, secondId, payload }) {
      //   const response = await moduleApi.updateById(firstId, secondId, payload);
      //   commit({ type: 'setById', data: response.data });
      // },
      async create({ commit }, {firstId, payload }) {
        const response = await moduleApi.create(firstId, payload);
        commit({ type: 'setById', data: response.data });
        return response;
      },
      async deleteById({ commit }, { firstId, secondId }) {
        await moduleApi.deleteById(firstId, secondId);
        commit({ type: 'removeById', firstId, secondId });
      },
    },
  };

  return nested ? nestedStore : normalStore;

}

